/*
    Project: TCTEMP
    Version: V1.20
    File   : na_rdtsc.c

    Copyright (c) 2006-2010 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "na.h"

#pragma check_stack(off)
#pragma runtime_checks("", off)

#if defined(_M_IX86)

unsigned __int64 rdtsc(void)
{
    ULARGE_INTEGER uli;

    __try {
        __asm {
            xor     eax, eax
            cpuid
            rdtsc
            mov     uli.LowPart, eax
            mov     uli.HighPart, edx
        }
    } __except(EXCEPTION_EXECUTE_HANDLER) {
        uli.QuadPart = 0;
    }

    return uli.QuadPart;
}

#elif defined (_M_AMD64)

#pragma intrinsic(__cpuid)
extern void __cpuid(int rgiCPUInfo[4], int iInfoType);

#pragma intrinsic(__rdtsc)
extern unsigned __int64 __rdtsc(void);

unsigned __int64 rdtsc(void)
{
    int              rgiCPUInfo[4];
    unsigned __int64 ull;

    __try {
        __cpuid(rgiCPUInfo, 0);
        ull = __rdtsc();
    } __except(EXCEPTION_EXECUTE_HANDLER) {
        ull = 0;
    }

    return ull;
}

#else

#  error "Unsupported architecture"

#endif

#pragma runtime_checks("", restore)
#pragma check_stack()
